/*
 * AffineCCodeGenerator.java: This file is part of the SDSLC project.
 *
 * SDSLC: A compiler for high performance stencil computations
 *
 * Copyright (C) 2011-2013 Ohio State University
 *
 * This program can be redistributed and/or modified under the terms
 * of the license specified in the LICENSE.txt file at the root of the
 * project.
 *
 * Contact: P Sadayappan <saday@cse.ohio-state.edu>
 */
package edu.osu.cse.hpcrl.sdsl.backend;


import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.HashMap;

import org.apache.log4j.Logger;

import edu.osu.cse.hpcrl.sdsl.ir.Add;
import edu.osu.cse.hpcrl.sdsl.ir.Assignment;
import edu.osu.cse.hpcrl.sdsl.ir.BinaryOp;
import edu.osu.cse.hpcrl.sdsl.ir.Computable;
import edu.osu.cse.hpcrl.sdsl.ir.Constant;
import edu.osu.cse.hpcrl.sdsl.ir.Divide;
import edu.osu.cse.hpcrl.sdsl.ir.DoubleLiteral;
import edu.osu.cse.hpcrl.sdsl.ir.EqualTo;
import edu.osu.cse.hpcrl.sdsl.ir.Expression;
import edu.osu.cse.hpcrl.sdsl.ir.ExpressionStatement;
import edu.osu.cse.hpcrl.sdsl.ir.FloatLiteral;
import edu.osu.cse.hpcrl.sdsl.ir.FunctionCall;
import edu.osu.cse.hpcrl.sdsl.ir.GreaterEqThan;
import edu.osu.cse.hpcrl.sdsl.ir.GreaterThan;
import edu.osu.cse.hpcrl.sdsl.ir.Grid;
import edu.osu.cse.hpcrl.sdsl.ir.GridData;
import edu.osu.cse.hpcrl.sdsl.ir.GridDataRef;
import edu.osu.cse.hpcrl.sdsl.ir.IntLiteral;
import edu.osu.cse.hpcrl.sdsl.ir.LessEqThan;
import edu.osu.cse.hpcrl.sdsl.ir.LessThan;
import edu.osu.cse.hpcrl.sdsl.ir.LocalVar;
import edu.osu.cse.hpcrl.sdsl.ir.LocalVarRef;
import edu.osu.cse.hpcrl.sdsl.ir.LongLiteral;
import edu.osu.cse.hpcrl.sdsl.ir.Multiply;
import edu.osu.cse.hpcrl.sdsl.ir.NotEqualTo;
import edu.osu.cse.hpcrl.sdsl.ir.Parameter;
import edu.osu.cse.hpcrl.sdsl.ir.PointFunction;
import edu.osu.cse.hpcrl.sdsl.ir.PointFunctionCall;
import edu.osu.cse.hpcrl.sdsl.ir.PointFunctionStatement;
import edu.osu.cse.hpcrl.sdsl.ir.Reduction;
import edu.osu.cse.hpcrl.sdsl.ir.ReductionStatement;
import edu.osu.cse.hpcrl.sdsl.ir.SDSLProgram;
import edu.osu.cse.hpcrl.sdsl.ir.SpaceRef;
import edu.osu.cse.hpcrl.sdsl.ir.Statement;
import edu.osu.cse.hpcrl.sdsl.ir.Stencil;
import edu.osu.cse.hpcrl.sdsl.ir.StencilStatement;
import edu.osu.cse.hpcrl.sdsl.ir.Subregion;
import edu.osu.cse.hpcrl.sdsl.ir.Subtract;
import edu.osu.cse.hpcrl.sdsl.ir.UnaryMinus;

// FIXME: This file is an atrocious hack..
// It really needs to be cleaned up and functionality shared with the 
// overtile code generator should be moved to the superclass. Also,
// documentation is horrid and should be improved. Finally, it should really
// log what it is doing.
/**
 * @author Tom Henretty <henretty@cse.ohio-state.edu>
 */
public class AffineCCodeGenerator implements CodeGenerator {
  
  private static Logger log = Logger.getLogger(AffineCCodeGenerator.class.getName());
  
  private static final String EOL = System.getProperty("line.separator");
  private static final String TAB = "  ";
  
  private static final String t = "t";
  private static final String[] spaceIters = {"i","j","k"};
  
  private SDSLProgram p;
  private String source;
  private String functionName;
  private int maxDim = 0;

  public AffineCCodeGenerator() {
    super();
  }
  
  private void initCodegen(SDSLProgram p) {
    this.p = p;
    this.source = p.getSDSLText();
    this.functionName = genFunctionName();
  }
  
  /**
   * Generates a C function prototype for the given SDSLProgram.
   * 
   * @param p  The SDSLProgram IR to generate a function prototype for.
   */
  public String generateCFunctionPrototype(SDSLProgram p) {
    StringBuffer out = new StringBuffer();
    initCodegen(p);
    
    String proto = "";
    
    // Name and return type
    proto += "int " + functionName + "(";
    maxDim = 0;
    
    // Parameter for each grid data
    boolean firstParam = true;
    for (GridData gd : p.getGridData()) {
      int dims = gd.getDefiningGrid().getNumDims();
      maxDim = maxDim > dims ? maxDim : dims;
      String comma = firstParam ? "" : ", ";
      proto += comma + gd.getType() + " *p_" + gd.getName();
      firstParam = false;
    }
    
    // Parameter for each SDSL parameter
    for (Parameter param : p.getParams()) {
      proto += ", " + param.getType() + " sdsl_" + param.getName();
    }
    proto += ")";
    out.append(proto);
    return out.toString();
  }

  /**
   * Generates a C function call for the given SDSLProgram.
   * 
   * @param p  The SDSLProgram IR to generate a function call for.
   */
  public String generateCFunctionCall(SDSLProgram p) {
    StringBuffer out = new StringBuffer();
    initCodegen(p);
    
    // Function name
    String call = "const int sdsl_return_" + p.getSDSLIndex() + " = " + functionName + "(";
    
    // Grid data
    boolean firstArg = true;
    for (GridData gd : p.getGridData()) {
      String comma = firstArg ? "" : ", ";
      call += comma + gd.getName();
      firstArg = false;
    }
    
    // Parameters
    for (Parameter param : p.getParams()) {
      call += ", " + param.getName();
    }
    
    call += ");" + EOL;
    out.append(call);
    
    return out.toString();
  }

  public String generateCFunctionDefinition(SDSLProgram p) {
    StringBuffer out = new StringBuffer();
    initCodegen(p);
    if (p.getCLOptions().isExternC()) {
      out.append("#ifdef __cplusplus\nextern \"C\"\n#endif\n");
    }
    out.append(generateCFunctionPrototype(p));
    out.append(" {" + EOL);
    out.append(genCasts());
    out.append(genArrayDecls());
    out.append(genConstants());
    out.append(genReductionVars());
    out.append(genPFVars());
    out.append(genInitialize());
    out.append(genIterativeLoop());
    out.append(genFinalize());
    out.append(EOL + "return 0;}" + EOL);
    return out.toString();
  }
  
  public String generateCPreprocessorInfo() {
    StringBuffer out = new StringBuffer();
    out.append("#include <float.h>" + EOL);
    out.append("#include <math.h>" + EOL);
    out.append("#include <stdio.h>" + EOL);
    out.append("#include <stdlib.h>" + EOL);
    out.append("#include <string.h>" + EOL);
    // FIXME: Resolve conflict with max/min in C++ algorithm header
    out.append("#define min(a,b) (a < b ? a : b)" + EOL);
    out.append("#define max(a,b) (a > b ? a : b)" + EOL);
    return out.toString();
  }
  
  /**
   * Casts pointers to multidimensional array references
   */
  private String genCasts() {
    StringBuffer out = new StringBuffer();
    for (GridData gd : p.getGridData()) {
      // Get the defining grid and size information
      Grid grid = gd.getDefiningGrid();
      int numDims = grid.getNumDims();
      ArrayList<Subregion> dimSizes = grid.getBounds().getSubregions();
      
      // Build multidim array declaration
      String cast = TAB + gd.getType() + " (*" + gd.getName() + ")";
      for (int dim = 1; dim < numDims; dim++) {
        cast += "[" + exprToString(dimSizes.get(dim).getUB()) + "]";
      }
      
      // Build parameter cast
      cast += " = (" + gd.getType() + " (*)";
      for (int dim = 1; dim < numDims; dim++) {
        cast += "[" + exprToString(dimSizes.get(dim).getUB()) + "]";
      }
      cast += ")p_" + gd.getName() + ";" + EOL;
      out.append(cast);
    }
    return out.toString();
  }
  
  /**
   * Generate array declarations for grid data defined on multiple timesteps 
   */
  private String genArrayDecls() {
    StringBuffer out = new StringBuffer();
    String decls = "";
    for (GridData gd : p.getGridData()) {
      // Get the defining grid and size information
      Grid grid = gd.getDefiningGrid();
      int numDims = grid.getNumDims();
      ArrayList<Subregion> dimSizes = grid.getBounds().getSubregions();
      int timeDimSize = gd.getDefinedTimes().size();
      
      if (timeDimSize == 2) {
        // Generate multidim array declaration
        decls += TAB + gd.getType() + " (*" + gd.getName() + "_old)";
        for (int dim = 1; dim < numDims; dim++) {
          decls += "[" + exprToString(dimSizes.get(dim).getUB()) + "]";
        }
        
        // Generate multidim array definition
        decls += " = " + "(" + gd.getType() + " (*)";
        for (int dim = 1; dim < numDims; dim++) {
          decls += "[" + exprToString(dimSizes.get(dim).getUB()) + "]";
        }
        
        // Generate malloc
        decls += ")malloc(";
        for (int dim = 0; dim < numDims; dim++) {
          decls += exprToString(dimSizes.get(dim).getUB()) + "*";
        }
        decls += "sizeof(" + gd.getType() + "));" + EOL;
      }
    }
    out.append(decls);
    return out.toString();
  }
  
  /**
   * Copy original grid data contents into all arrays representing that grid 
   * data. 
   */
  private String genArrayCopies() {
    StringBuffer out = new StringBuffer();
    out.append(EOL + indent(2) + "// [SDSL] Begin array copies" + EOL);
    String copies = "";
    int depth;
    for (depth = 0; depth < maxDim; depth++) {
      String iter = spaceIters[depth];
      Subregion sr = p.getGrids().get(0).getBounds().getSubregions().get(depth);
      copies += indent(depth+2) + "for (" + iter + " = " + 0 + "; " +
                                    iter + " < " + exprToString(sr.getUB()) + "; " +iter + "++) {" + EOL;
    }
    
    for (GridData gd : p.getGridData()) {
      int timeDimSize = gd.getDefinedTimes().size();
      if (timeDimSize == 2) {
        copies += indent(depth+2) + gd.getName() + "_old";
        for (int dim = 0; dim < gd.getDefiningGrid().getNumDims(); dim++) {
          copies += "[" + spaceIters[dim] + "]";
        }
        copies += " = " + gd.getName();
        for (int dim = 0; dim < gd.getDefiningGrid().getNumDims(); dim++) {
          copies += "[" + spaceIters[dim] + "]";
        }
        copies += ";" + EOL;
      }
    }
    
    
    for (; depth > 0; depth--) {
      copies += indent(depth+1) + "}" + EOL;
    }
    out.append(copies);
    out.append(indent(2) + "// [SDSL] End array copies" + EOL + EOL);
    return out.toString();
  }
  
  /**
   * Generate C language versions of SDSL constants.
   */
  private String genConstants() {
    StringBuffer out = new StringBuffer();
    for (Constant cst : p.getConsts()) {
      out.append(TAB + "const " + cst.getType() + " sdsl_" + cst.getName() + 
                 " = " + exprToString(cst.getInitExpr()) + ";" + EOL);
    }
    return out.toString();
  }
  
  /**
   * Generate C language versions of reduction variables.
   */
  private String genReductionVars() {
    StringBuffer out = new StringBuffer();
    for (Computable c : p.getIteration().getComputables()) {
      if (c instanceof Reduction) {
        Reduction r = (Reduction)c;
        String assocOp = r.getAssocOp();
        // FIXME: Need to determine type based on expressions in reduction
        String type = "double";
        out.append(TAB + type + " sdsl_" + r.getName() + " = ");
        if (assocOp.equals("max")) {
          out.append("-DBL_MAX;" + EOL);
        } else if (assocOp.equals("min")) {
          out.append("DBL_MAX;" + EOL);
        } else if (assocOp.equals("+")) {
          out.append("0.0;" + EOL);
        } else if (assocOp.equals("*")) {
          out.append("1.0;" + EOL);
        }
      }
    }
    return out.toString();
  }
  
  /**
   * Generate C language versions of point function local variables.
   */
  private String genPFVars() {
    StringBuffer out = new StringBuffer();
    String pfVars = "";
    for (PointFunction pf : p.getPointFunctions()) {
      for (LocalVar var : pf.getLocalVars()) {
        pfVars += TAB + var.getType() + " sdsl_" + var.getName() + ";" + EOL;
      }
    }
    out.append(pfVars);
    return out.toString();
  }
  
  private String genInitialize() {
    StringBuffer out = new StringBuffer();
    String loopVars = TAB + "int " + t;
    for (int i = 0; i < maxDim; i++) {
      loopVars += ", " + spaceIters[i];
    }
    //for (String iter : spaceIters) {
    //  loopVars += ", " + iter;
    //}
    loopVars += ";" + EOL;
    out.append(loopVars);
    return out.toString();
  }
  
  /**
   * Generates C code for the iterative loop nest.
   */
  private String genIterativeLoop() {
    StringBuffer out = new StringBuffer();
    long maxIter = p.getMaxIterations();
    long checkEvery = p.getIteration().getCheckEvery();
    boolean hasCheck = p.getIteration().getConvergeCheck() != null;
    boolean timeTiled = checkEvery > 1  && hasCheck;
    // FIXME: Hack, need a better way to determine if a check every clause is 
    // present and consider GPU time tiling
    if (checkEvery == 0) checkEvery = 1;
    
    if (timeTiled) {
      // Generate outer time tile loop
      out.append(TAB + "int tt;");
      out.append(TAB + "for(tt = 0; tt < " + maxIter + "; tt += " + checkEvery + ") {\n");
      // Generate inner time loop
      out.append(TAB + "#pragma scop" + EOL);
      out.append(TAB + "for (" + t + " = tt; " 
    			+ t + " < min(tt+" + checkEvery + ", " + maxIter + "); " 
    			+ t + "++) {" + EOL);
    }
    else {
      // Generate time loop
      if (!hasCheck) {
        out.append(TAB + "#pragma scop" + EOL);
      }
      out.append(TAB + "for (" + t + " = 0; " 
      			+ t + " < " + maxIter + "; " 
      			+ t + "++) {" + EOL);
    }
    out.append(genArrayCopies());
    for (Computable c : p.getIteration().getComputables()) {
      if (c instanceof Stencil) {
        out.append(genStencil((Stencil)c));
      }
    }
    if (!timeTiled) {
      // Generate reductions
      for (Computable c : p.getIteration().getComputables()) {
        if (c instanceof Reduction) {
          out.append(genReduction((Reduction)c));
        }
      }
      if (hasCheck) {
        out.append(TAB + genCheck());
      }
    }
    out.append(TAB + "}" + EOL); // End time loop
    // Avoids including a break statement from a converge check in a scop
    if (!(hasCheck && !timeTiled)) {
      out.append(TAB + "#pragma endscop" + EOL);
    }
    
    if (timeTiled) {
      // Generate reductions
      for (Computable c : p.getIteration().getComputables()) {
        if (c instanceof Reduction) {
          out.append(genReduction((Reduction)c));
        }
      }
      if (hasCheck) {
        out.append(TAB + genCheck());
      }
      out.append(TAB + "}" + EOL); // End time tile loop
    }
    return out.toString();
  }
  
  /**
   * Generates C code for an individual stencil.
   * 
   * @param s
   */
  private String genStencil(Stencil s) {
    StringBuffer out = new StringBuffer();
    int depth = 0;
    
    ArrayList<StencilStatement> ssList = s.getStatements();
    out.append(indent(depth+2) + "// [SDSL] Begin stencil" + EOL);
    for (StencilStatement ss : ssList) {
      ArrayList<Subregion> subregions = ss.getRegion().getSubregions();
      
      // Loop nests
      for (Subregion sr : subregions) {
        String iter = spaceIters[depth];
        out.append(indent(depth+2) + "for (" + iter + " = " + exprToString(sr.getLB()) + "; " +
                                      iter + " <= " + exprToString(sr.getUB()) + "; " +iter + "++) {" + EOL);
        depth++;
      }
      
      // Statements
      Statement st = ss.getStatement();
      PointFunctionCall pfc = null;
      ExpressionStatement exs = null;
      if (st instanceof ExpressionStatement) {
        exs = (ExpressionStatement)st;
        Expression ex = exs.getExpression();
        if (ex instanceof PointFunctionCall) {
          pfc = (PointFunctionCall)ex;
        }
      }
      
      if (pfc != null) {
        out.append(genPFCall(pfc));
      } else if (exs != null) {
        out.append(indent(depth+2) + exprToString(exs.getExpression()) + ";");
        //out.append(pfc.toString(indent(depth+2)));
      } else {
        out.append("/* Unsupported Statement */" + EOL);
      }
      
      // Close loop nests
      for (Subregion sr : subregions) {
        depth--;
        out.append(EOL + indent(depth+2) + "}");
      }
      out.append(EOL);
    }
    out.append(indent(depth+2) + "// [SDSL] End stencil" + EOL + EOL);
    return out.toString();
  }
  
  private String genReduction(Reduction r) {
    StringBuffer out = new StringBuffer();    
    int depth = 0;

    String assocOp = r.getAssocOp();
    String name = r.getName();

    ArrayList<ReductionStatement> rsList = r.getStatements();
    out.append(indent(depth+2) + "// [SDSL] Begin reduction" + EOL);
    // FIXME: Hardcoded reduction var type to double. There needs to be a way
    // to determine what this type should be.
    out.append(indent(depth+2) + "sdsl_" + name + " = ");
    if (assocOp.equals("max")) {
      out.append("-DBL_MAX;" + EOL);
    } else if (assocOp.equals("min")) {
      out.append("DBL_MAX;" + EOL);
    } else if (assocOp.equals("+")) {
      out.append("0.0;" + EOL);
    } else if (assocOp.equals("*")) {
      out.append("1.0;" + EOL);
    }
    for (ReductionStatement rs : rsList) {
      ArrayList<Subregion> subregions = rs.getRegion().getSubregions();

      // Loop nests
      for (Subregion sr : subregions) {
        String iter = spaceIters[depth];
        out.append(indent(depth+2) + "for (" + iter + " = " + exprToString(sr.getLB()) + "; " +
            iter + " <= " + exprToString(sr.getUB()) + "; " + iter + "++) {" + EOL);
        depth++;
      }

      // Statements
      if (assocOp.equals("+") || assocOp.equals("*")) {
        out.append(indent(depth+2) + "sdsl_" + name + " = " + "sdsl_" + name + " " + assocOp + exprToString(rs.getExpression()) + ";");
      }
      else if (assocOp.equals("max") || assocOp.equals("min")) {
        out.append(indent(depth+2) + "sdsl_" + name + " = " + assocOp + "(" + "sdsl_" + name + ", " + exprToString(rs.getExpression()) + ");");
      }

      // Close loop nests
      for (Subregion sr : subregions) {
        depth--;
        out.append(EOL + indent(depth+2) + "}");
      }
      out.append(EOL);
    }
    out.append(indent(depth+2) + "// [SDSL] End reduction" + EOL);
    return out.toString();

  }
  
  private String genCheck() {
    StringBuffer out = new StringBuffer();
    Expression check = p.getIteration().getConvergeCheck();
    if (check != null) {
      out.append("if (" + exprToString(check) + ") break;" + EOL);
    }
    return out.toString();
  }
  
  /**
   * Generates code for a point function call.
   * 
   * @param pfc
   */
  private String genPFCall(PointFunctionCall pfc) {
    StringBuffer out = new StringBuffer();
    String pfCode = "";
    
    // Generate parameter bindings
    ArrayList<GridData> args = pfc.getArgs();
    ArrayList<GridData> params = pfc.getPointFunction().getParams();
    HashMap<String,GridData> paramMap = new HashMap<String,GridData>();
    int i = 0;
    for (GridData g : params) {
      paramMap.put(g.getName(),args.get(i));
      i++;
    }
    
    // Generate statement code
    for (PointFunctionStatement pfs : pfc.getPointFunction().getStmts()) {
      Statement s = pfs.getStatement();
      if (s instanceof LocalVar) {
        LocalVar lv = (LocalVar)s;
        Expression init = lv.getInitExpr();
        if (init != null) {
          pfCode += indent(5) + "sdsl_" + lv.getName() + " = " + exprToString(init,paramMap) + ";" + EOL;
        }
      } else if (s instanceof ExpressionStatement) {
        ExpressionStatement es = (ExpressionStatement)s;
        Expression e = es.getExpression();
        if (e instanceof Assignment) {
          Assignment a = (Assignment)e;
          Expression lhs = a.getLHS();
          Expression rhs = a.getRHS();
          pfCode += indent(5) + exprToString(lhs,paramMap) + " = " + exprToString(rhs,paramMap) + ";" + EOL;
        }
      }
    }
    out.append(pfCode);
    return out.toString();
  }
  
  /**
   * Converts an Expression in SDSL IR to a C string representation.
   * 
   * @param e  The expression
   * @return  The expression as a C string
   */
  private String exprToString(Expression e) {
    return exprToString(e, new HashMap<String,GridData>());
  }
  
  /**
   * Converts an expression in SDSL IR to a C string representation. Uses 
   * Parameter bindings so that expressions from inside point functions can 
   * be converted depending on the circumstances of their call.
   * 
   * @param e  The expression
   * @param bindings  Bindings for PointFunction grid data
   * @return The expression as a C string
   */
  private String exprToString(Expression e, HashMap<String,GridData> bindings) {
    String eStr = "";
    if (e instanceof GridDataRef) {
      GridDataRef gdr = (GridDataRef)e;
      GridData gd = gdr.getGridData();
      String name = gdr.getGridData().getName();
      if (bindings.containsKey(name)) {
        gd = bindings.get(name);
        name = gd.getName();  
      }
      eStr += name;
      
      int gdSize = 0;
      for (GridData gdp : p.getGridData()) {
        if (gdp.getName().equals(gd.getName())) {
          gdSize = gdp.getDefinedTimes().size();
          break;
        }
      }
      
      Expression tre = gdr.getTimeRef();
      int tri = 0;
      // FIXME: Check for expressions, long literals
      if (tre instanceof IntLiteral) {
        tri = ((IntLiteral)tre).getValue();
      }
      if (gdSize > 1 && tri == 0) {
        eStr += "_old";
      }
      
      int i = 0;
      for (SpaceRef sr : gdr.getSpaceRefs()) {
        eStr += "[" + spaceIters[i] + "+" +  exprToString(sr.getRef(),bindings) + "]";
        i++;
      }
    } else if (e instanceof IntLiteral) {
      eStr += ((IntLiteral)e).getValue();
    } else if (e instanceof LongLiteral) {
      eStr += ((LongLiteral)e).getValue();
    } else if (e instanceof FloatLiteral) {
      eStr += ((FloatLiteral)e).getValue();
    } else if (e instanceof DoubleLiteral) {
      eStr += ((DoubleLiteral)e).getValue();
    } else if (e instanceof UnaryMinus) {
      eStr += "(-";
      eStr += ((UnaryMinus)e).getOperand();
      eStr += ")";
    } else if (e instanceof Add) {
      BinaryOp o = (BinaryOp)e;
      eStr += "(";
      eStr += exprToString(o.getLHS(), bindings);
      eStr += "+";
      eStr += exprToString(o.getRHS(), bindings);
      eStr += ")";
    } else if (e instanceof Subtract) {
      BinaryOp o = (BinaryOp)e;
      eStr += "(";
      eStr += exprToString(o.getLHS(), bindings);
      eStr += "-";
      eStr += exprToString(o.getRHS(), bindings);
      eStr += ")";
    } else if (e instanceof Multiply) {
      BinaryOp o = (BinaryOp)e;
      eStr += "(";
      eStr += exprToString(o.getLHS(), bindings);
      eStr += "*";
      eStr += exprToString(o.getRHS(), bindings);
      eStr += ")";
    } else if (e instanceof Divide) {
      BinaryOp o = (BinaryOp)e;
      eStr += "(";
      eStr += exprToString(o.getLHS(), bindings);
      eStr += "/";
      eStr += exprToString(o.getRHS(), bindings);
      eStr += ")";
    } else if (e instanceof GreaterThan) {
      BinaryOp o = (BinaryOp)e;
      eStr += "(";
      eStr += exprToString(o.getLHS(), bindings);
      eStr += ">";
      eStr += exprToString(o.getRHS(), bindings);
      eStr += ")";
    } else if (e instanceof LessThan) {
      BinaryOp o = (BinaryOp)e;
      eStr += "(";
      eStr += exprToString(o.getLHS(), bindings);
      eStr += "<";
      eStr += exprToString(o.getRHS(), bindings);
      eStr += ")";
    } else if (e instanceof GreaterEqThan) {
      BinaryOp o = (BinaryOp)e;
      eStr += "(";
      eStr += exprToString(o.getLHS(), bindings);
      eStr += ">=";
      eStr += exprToString(o.getRHS(), bindings);
      eStr += ")";
    } else if (e instanceof LessEqThan) {
      BinaryOp o = (BinaryOp)e;
      eStr += "(";
      eStr += exprToString(o.getLHS(), bindings);
      eStr += "<=";
      eStr += exprToString(o.getRHS(), bindings);
      eStr += ")";
    } else if (e instanceof EqualTo) {
      BinaryOp o = (BinaryOp)e;
      eStr += "(";
      eStr += exprToString(o.getLHS(), bindings);
      eStr += "==";
      eStr += exprToString(o.getRHS(), bindings);
      eStr += ")";
    } else if (e instanceof NotEqualTo) {
      BinaryOp o = (BinaryOp)e;
      eStr += "(";
      eStr += exprToString(o.getLHS(), bindings);
      eStr += "!=";
      eStr += exprToString(o.getRHS(), bindings);
      eStr += ")";
    } else if (e instanceof LocalVarRef) {
      LocalVarRef LVR = (LocalVarRef)e;
      eStr += "sdsl_" + LVR.getName();;
    } else if (e instanceof Assignment) {
      BinaryOp o = (BinaryOp)e;
      eStr += exprToString(o.getLHS(), bindings);
      eStr += " = ";
      eStr += exprToString(o.getRHS(), bindings);
    } else if (e instanceof FunctionCall) {
      FunctionCall FC = (FunctionCall)e;
      // HACK
      if (FC.getName().equals("rsqrt")) {
        eStr += "(1.0/sqrt(";
      } else if (FC.getName().equals("rsqrtf")) {
    	eStr += "(1.0f/sqrt(";
      } else {
        eStr += "(" + FC.getName() + "(";
      }
      // END HACK
      ArrayList<Expression> Params = FC.getArgs();
      for (int i = 0, s = Params.size(); i != s; ++i) {
        if (i > 0)
          eStr += ", ";
        eStr += exprToString(Params.get(i), bindings);
      }
      eStr += "))";
    } else {
      eStr += "/* Unhandled expression */";
    }
    return eStr;
  }
  
  /**
   * Frees any arrays we created to represent multiple timesteps of the same 
   * grid data.
   */
  private String genFinalize() {
    StringBuffer out = new StringBuffer();
    String frees = "";
    for (GridData gd : p.getGridData()) {
      int timeDimSize = gd.getDefinedTimes().size();
      if (timeDimSize == 2) {
        frees += TAB + "free(" + gd.getName() + "_old);" + EOL;
      }
    }
    out.append(frees);
    return out.toString();
  }
  
  
  private String indent(int level) {
    StringBuffer sb = new StringBuffer();
    for (int i = 0; i < level; i++) {
      sb.append(TAB);
    }
    return sb.toString();
  }
  
  private String genFunctionName() {
    MessageDigest md = null;
    String name = "sdsl_program";
    try {
      md = MessageDigest.getInstance("SHA1");
      name = "sdsl_program_" + byteArrayToHexString(md.digest(source.getBytes()));
    } catch (NoSuchAlgorithmException E) {
    }
    
    return name;
  }
  
  // http://stackoverflow.com/questions/4895523/java-string-to-sha1
  private static String byteArrayToHexString(byte[] b) {
    String result  = "";
    for (int i = 0; i < b.length; i++) {
          result  += Integer.toString((b[i] & 0xff) + 0x100, 16).substring(1);
    }
    return result;
  }
}
