/*
 * CodeGenerator.java: This file is part of the SDSLC project.
 *
 * SDSLC: A compiler for high performance stencil computations
 *
 * Copyright (C) 2011-2013 Ohio State University
 *
 * This program can be redistributed and/or modified under the terms
 * of the license specified in the LICENSE.txt file at the root of the
 * project.
 *
 * Contact: P Sadayappan <saday@cse.ohio-state.edu>
 */
package edu.osu.cse.hpcrl.sdsl.backend;

import edu.osu.cse.hpcrl.sdsl.ir.SDSLProgram;

/**
 * Interface to be implemented by SDSL code generators. It is assumed  
 * that the generated code will replace an SDSL section embedded in C/C++ code.
 * 
 * Further, it is assumed that the SDSL section will be replaced by a forward
 * declaration of a C function containing optimized code, along with a call to 
 * that function.
 * 
 * Finally, it is assumed that the C function definition will occur at the end 
 * of the file containing embedded SDSL.
 * 
 * @author Tom Henretty <henretty@cse.ohio-state.edu>
 *
 */
public interface CodeGenerator {
  
  // FIXME: Describe parameters, griddata, timesteps, dimensions and 
  //        their interaction with surrounding C code.
  /**
   * Generates a function prototype for the given SDSLProgram in the C 
   * language.
   * 
   * @param p  The SDSLProgram to generate a prototype for.
   * @return A String representation of the generated C function prototype.
   */
  public String generateCFunctionPrototype(SDSLProgram p);
  
  /**
   * Generates a C language call to the function declared by
   * generateCFunctionPrototype().
   * 
   * @param p  The SDSLProgram to generate a call for.
   * @return A String representation of the generated C function call.
   */
  public String generateCFunctionCall(SDSLProgram p);
  
  /**
   * Generates a C language definition of the function declared by
   * generateCFunctionPrototype().
   * 
   * @param p  The SDSLProgram to generate a function definition for.
   * @return A String representation of the generated C function definition.
   */
  public String generateCFunctionDefinition(SDSLProgram p);
  
  /**
   * Generates code used by the C/C++ preprocessor including #include and 
   * #define statements.
   * 
   * @return A String with preprocessor info  
   */
  public String generateCPreprocessorInfo();
}
