/*
 * LibOvertileWrapper.java: This file is part of the SDSLC project.
 *
 * SDSLC: A compiler for high performance stencil computations
 *
 * Copyright (C) 2011-2013 Ohio State University
 *
 * This program can be redistributed and/or modified under the terms
 * of the license specified in the LICENSE.txt file at the root of the
 * project.
 *
 * Contact: P Sadayappan <saday@cse.ohio-state.edu>
 */
/**
 * 
 */
package edu.osu.cse.hpcrl.sdsl.backend;


/**
 * Uses libovertile to generate CUDA device and C host functions from the SSP 
 * DSL.
 * 
 * @author Tom Henretty <henretty@cse.ohio-state.edu>
 *
 */
public final class LibOvertileWrapper {
  static {System.loadLibrary("overtile");}
  /**
   * Generates GPU device and host code from SSP DSL.
   * 
   * @param ssp  SSP DSL text
   * @param threadBlockSizes  Array of GPU thread block sizes
   * @param spaceTileSizes  Array of space tile sizes
   * @param timeTileSize  How many timesteps to calculate in each kernel invocation
   * @param legacy  If true code is generated for GT2xx/Tesla 10xx GPU. If false code generated for later devices.
   * @param verbose  Provide verbose messages during execution.
   * @return  A String containing a CUDA GPU kernel definition followed by a 
   *          host C function to call the previously defined kernel. 
   */
  public static native String generateCUDAFunctionDefinition(String ssp, 
                                                             int[] threadBlockSizes, 
                                                             int[] spaceTileSizes,
                                                             int timeTileSize,
                                                             boolean legacy,
                                                             boolean verbose);
}
