/*
 * FunctionCall.java: This file is part of the SDSLC project.
 *
 * SDSLC: A compiler for high performance stencil computations
 *
 * Copyright (C) 2011-2013 Ohio State University
 *
 * This program can be redistributed and/or modified under the terms
 * of the license specified in the LICENSE.txt file at the root of the
 * project.
 *
 * Contact: P Sadayappan <saday@cse.ohio-state.edu>
 */
package edu.osu.cse.hpcrl.sdsl.ir;

import java.util.ArrayList;
import java.util.HashMap;

/**
 * @author Tom Henretty <henretty@cse.ohio-state.edu>
 */
public class FunctionCall extends Expression {
  private String name;
  private ArrayList<Expression> args;
  
  /**
   *
   */
  public FunctionCall() {
    super();
    args = new ArrayList<Expression>();
    name = "";
  }
  
  /**
   *
   */
  public FunctionCall(String name) {
    super();
    args = new ArrayList<Expression>();
    this.name = name;
  }
  
  /**
   *
   */
  public FunctionCall(String name, ArrayList<Expression> args) {
    super();
    this.args = args;
    this.name = name;
  }
  
  /**
   *
   */
  public String getName() {
    return name;
  }
  
  /**
   *
   */
  public void setName(String name) {
    this.name = name;
  }
  
  /**
   *
   */
  public ArrayList<Expression> getArgs() {
    return args;
  }
  
  /**
   *
   */
  public void addArg(Expression arg) {
    args.add(arg);
  }
  
  /**
   *
   */
  public void setArgs(ArrayList<Expression> args) {
    this.args = args;
  }
  
  public void bindParameters(HashMap<String,GridData> map) {
    for (Expression e : args) {
      e.bindParameters(map);
    }
  }
  
  /**
   *
   */
  public String toString() {
    int i = 0;
    StringBuffer out = new StringBuffer();
    out.append(name + "(");
    for (Expression e : args) {
      i++;
      out.append(e);
      if (i < args.size()) {
        out.append(",");
      }
    };
    out.append(")");
    return out.toString();
  }
  
  /**
   *
   */
  public String toNoParenString() {
    return toString();
  }
}