/*
 * GridData.java: This file is part of the SDSLC project.
 *
 * SDSLC: A compiler for high performance stencil computations
 *
 * Copyright (C) 2011-2013 Ohio State University
 *
 * This program can be redistributed and/or modified under the terms
 * of the license specified in the LICENSE.txt file at the root of the
 * project.
 *
 * Contact: P Sadayappan <saday@cse.ohio-state.edu>
 */
package edu.osu.cse.hpcrl.sdsl.ir;

import java.util.ArrayList;

/**
 * @author Tom Henretty <henretty@cse.ohio-state.edu>
 */
public class GridData {
  private String name;
  private String type;
  private Grid definingGrid;
  
  private ArrayList<Expression> definedTimes;
  
  /**
   *
   */
  public GridData() {
    super();
    this.name = null;
    this.type = null;
    this.definingGrid = null;
    this.definedTimes = new ArrayList<Expression>();
  }
  
  /**
   *
   */
  public GridData(String name) {
    super();
    this.type = null;
    this.definingGrid = null;
    this.name = name;
    this.definedTimes = new ArrayList<Expression>();
  }
  
  /**
   *
   */
  public GridData(String name, String type, Grid g) {
    super();
    this.name = name;
    this.type = type;
    this.definingGrid = g;
    this.definedTimes = new ArrayList<Expression>(); 
  }
  
  /**
   *
   */
  public GridData(String name, String type, Grid g, ArrayList<Expression> definedTimes) {
    super();
    this.name = name;
    this.type = type;
    this.definingGrid = g;
    this.definedTimes = definedTimes; 
  }
  
  /**
   *
   */
  public String getName() {
    return name;
  }
  
  /**
   *
   */
  public void setName(String name) {
    this.name = name;
  }
  
  /**
   *
   */
  public String getType() {
    return type;
  }
  
  /**
   *
   */
  public void setType(String type) {
    this.type = name;
  }
  
  /**
   *
   */
  public Grid getDefiningGrid() {
    return definingGrid;
  }
  
  /**
   *
   */
  public void setDefiningGrid(Grid g) {
    this.definingGrid = g;
  }
  
  /**
   *
   */
  public void addDefinedTime(int time) {
    definedTimes.add(new IntLiteral(time));
  }
  
  /**
   *
   */
  public ArrayList<Expression> getDefinedTimes() {
    return definedTimes;
  }
  
}