/*
 * GridDataRef.java: This file is part of the SDSLC project.
 *
 * SDSLC: A compiler for high performance stencil computations
 *
 * Copyright (C) 2011-2013 Ohio State University
 *
 * This program can be redistributed and/or modified under the terms
 * of the license specified in the LICENSE.txt file at the root of the
 * project.
 *
 * Contact: P Sadayappan <saday@cse.ohio-state.edu>
 */
package edu.osu.cse.hpcrl.sdsl.ir;

import java.util.ArrayList;
import java.util.HashMap;

/**
 * @author Tom Henretty <henretty@cse.ohio-state.edu>
 */
public class GridDataRef extends Expression {
  private GridData gridData;
  private Expression timeRef;
  private ArrayList<SpaceRef> spaceRefs;
  
  /**
   *
   */
  public GridDataRef() {
    super();
    gridData = null;
    timeRef = null;
    spaceRefs = new ArrayList<SpaceRef>();
  }
  
  /**
   *
   */
  public GridDataRef(GridData gd, Expression timeRef, ArrayList<SpaceRef> spaceRefs) {
    super();
    this.gridData = gd;
    this.timeRef = timeRef;
    this.spaceRefs = spaceRefs;
  }
  
  /**
   *
   */
  public GridData getGridData() {
    return gridData;
  }
  
  /**
   *
   */
  public void setGridData(GridData gd) {
    this.gridData = gd;
  }
  
  /**
   *
   */
  public Expression getTimeRef() {
    return timeRef;
  }
  
  /**
   *
   */
  public void setTimeRef(Expression e) {
    this.timeRef = e;
  }
  
  /**
   *
   */
  public ArrayList<SpaceRef> getSpaceRefs() {
    return spaceRefs;
  }
  
  /**
   *
   */
  public void setSpaceRefs(ArrayList<SpaceRef> spaceRefs) {
    this.spaceRefs = spaceRefs;
  }
  
  public void bindParameters(HashMap<String,GridData> map) {
    if (gridData != null) {
      GridData newGd = map.get(gridData.getName());
      if (newGd != null) {
        gridData = newGd;
      }
    }
  }
  
  public String toString() {
    StringBuilder out = new StringBuilder();
    String[] spaceIters = {"_i","_j","_k"};
    if (gridData != null) {
      int i = spaceIters.length - spaceRefs.size();
      
      out.append(gridData.getName() + "[" + timeRef + "]");
      for (SpaceRef s: spaceRefs) {
        if (s.isConstant()) {
          out.append("[" + s.getRef() + "]");
        }
        else {
          out.append("[" + spaceIters[i] + " + " + s.getRef() + "]");
        }
        i++;
      }
    }
    return out.toString();
  }
  
  public String toNoParenString() {
    return toString();
  }
}
