/*
 * LocalVar.java: This file is part of the SDSLC project.
 *
 * SDSLC: A compiler for high performance stencil computations
 *
 * Copyright (C) 2011-2013 Ohio State University
 *
 * This program can be redistributed and/or modified under the terms
 * of the license specified in the LICENSE.txt file at the root of the
 * project.
 *
 * Contact: P Sadayappan <saday@cse.ohio-state.edu>
 */
package edu.osu.cse.hpcrl.sdsl.ir;

import java.util.HashMap;
/**
 * @author Tom Henretty <henretty@cse.ohio-state.edu>
 */
public class LocalVar extends Statement {
  private String name;
  private String type;
  private Expression initExpr;
  
  /**
   *
   */
  public LocalVar() {
    super();
    name = "";
    type = "";
    initExpr = null;
  }
  
  /**
   *
   */
  public LocalVar(String name, String type) {
    super();
    this.name = name;
    this.type = type;
    initExpr = null;
  }
  
  /**
   *
   */
  public LocalVar(String name, String type, Expression initExpr) {
    super();
    this.name = name;
    this.type = type;
    this.initExpr = initExpr;
  }
  
  /**
   *
   */
  public String getName() {
    return name;
  }
  
  /**
   *
   */
  public void setName(String name) {
    this.name = name;
  }
  
  /**
   *
   */
  public String getType() {
    return type;
  }
  
  /**
   *
   */
  public void setType(String type) {
    this.type = type;
  }
  
  /**
   *
   */
  public Expression getInitExpr() {
    return initExpr;
  }
  
  /**
   *
   */
  public void setInitExpr(Expression e) {
    this.initExpr = e;
  }
  
  public void bindParameters(HashMap<String,GridData> map) {
    if (initExpr != null) {
      initExpr.bindParameters(map);
    }
  }
  
  public String toString() {
    StringBuffer out = new StringBuffer();
    out.append(type + " " + name);
    if (initExpr != null) {
      out.append(" = " + initExpr);
    }
    out.append(";");
    return out.toString();
  }
}
