/*
 * PointFunction.java: This file is part of the SDSLC project.
 *
 * SDSLC: A compiler for high performance stencil computations
 *
 * Copyright (C) 2011-2013 Ohio State University
 *
 * This program can be redistributed and/or modified under the terms
 * of the license specified in the LICENSE.txt file at the root of the
 * project.
 *
 * Contact: P Sadayappan <saday@cse.ohio-state.edu>
 */
package edu.osu.cse.hpcrl.sdsl.ir;



import java.util.ArrayList;
import java.util.HashMap;
/**
 * @author Tom Henretty <henretty@cse.ohio-state.edu>
 */
public class PointFunction {
  private String name;
  
  ArrayList<GridData> params;
  ArrayList<LocalVar> localVars;
  ArrayList<PointFunctionStatement> stmts;
  
  /**
   *
   */
  public PointFunction() {
    super();
    params    = new ArrayList<GridData>();
    localVars = new ArrayList<LocalVar>();
    
    stmts = null;
  }
  
  /**
   *
   */
  public PointFunction(String name, ArrayList<GridData> params, ArrayList<PointFunctionStatement> stmts) {
    super();
    this.params = params;
    this.localVars = new ArrayList<LocalVar>();
    for (PointFunctionStatement s : stmts) {
      if (s.getStatement() instanceof LocalVar) {
        LocalVar lv = (LocalVar)s.getStatement();
        this.localVars.add(lv);
      }
    }
    this.name = name;
    this.stmts = stmts;
  }
  
  /**
   *
   */
  public String getName() {
    return name;
  }
  
  /**
   *
   */
  public void setName(String name) {
    this.name = name;
  }
  
  /**
   *
   */
  public void addParam(GridData gd) {
    params.add(gd);
  }
  
  /**
   *
   */
  public GridData getParam(String name) {
    GridData ret = null;
    for (GridData g : params) {
      if (g.getName().equals(name)) {
        ret = g;
        break;
      }
    }
    return ret;
  }
  
  /**
   *
   */
   public boolean isParam(String name) {
    boolean ret = false;
    for (GridData g : params) {
      if (g.getName().equals(name)) {
        ret = true;
        break;
      }
    }
    return ret;
   }
  
  /**
   *
   */
  public ArrayList<GridData> getParams() {
    return params;
  }
  
  /**
   *
   */
  public void addLocalVar(LocalVar lv) {
    localVars.add(lv);
  }
  
  /**
   *
   */
  public LocalVar getLocalVar(String name) {
    LocalVar ret = null;
    for (LocalVar l : localVars) {
      if (l.getName().equals(name)) {
        ret = l;
        break;
      }
    }
    return ret;
  }
  
  /**
   *
   */
   public boolean isLocalVar(String name) {
    boolean ret = false;
    for (LocalVar l : localVars) {
      if (l.getName().equals(name)) {
        ret = true;
        break;
      }
    }
    return ret;
   }
  
  /**
   *
   */
  public ArrayList<LocalVar> getLocalVars() {
    return localVars;
  }
  
  /**
   *
   */
  public void addStmt(PointFunctionStatement s) {
    stmts.add(s);
  }
  
  /**
   *
   */
  public ArrayList<PointFunctionStatement> getStmts() {
    return stmts;
  }
  
  public String toString(ArrayList<GridData> args, String indent) {
    HashMap<String,GridData> paramMap = new HashMap<String,GridData>();
    int i = 0;
    for (GridData g : params) {
      paramMap.put(g.getName(),args.get(i));
      i++;
    }
    
    StringBuffer out = new StringBuffer();
    for (PointFunctionStatement pfs : stmts) {
      pfs.bindParameters(paramMap);
      out.append(indent + pfs.toString() + "\n");
    }
    return out.toString();
  }
}