/*
 * PointFunctionCall.java: This file is part of the SDSLC project.
 *
 * SDSLC: A compiler for high performance stencil computations
 *
 * Copyright (C) 2011-2013 Ohio State University
 *
 * This program can be redistributed and/or modified under the terms
 * of the license specified in the LICENSE.txt file at the root of the
 * project.
 *
 * Contact: P Sadayappan <saday@cse.ohio-state.edu>
 */
package edu.osu.cse.hpcrl.sdsl.ir;

import java.util.ArrayList;

/**
 * @author Tom Henretty <henretty@cse.ohio-state.edu>
 */
public class PointFunctionCall extends Expression {
  private PointFunction pf;
  private String name;
  private ArrayList<GridData> args;
  
  /**
   *
   */
  public PointFunctionCall() {
    super();
    args = new ArrayList<GridData>();
    name = "";
  }
  
  /**
   *
   */
  public PointFunctionCall(String name) {
    super();
    args = new ArrayList<GridData>();
    this.name = name;
  }
  
  /**
   *
   */
  public PointFunctionCall(String name, ArrayList<GridData> args) {
    super();
    this.args = args;
    this.name = name;
  }
  
  /**
   *
   */
  public PointFunctionCall(String name, ArrayList<GridData> args, PointFunction pf) {
    super();
    this.args = args;
    this.name = name;
    this.pf = pf;
  }
  
  /**
   *
   */
  public String getName() {
    return name;
  }
  
  /**
   *
   */
  public void setName(String name) {
    this.name = name;
  }
  
  /**
   *
   */
  public ArrayList<GridData> getArgs() {
    return args;
  }
  
  /**
   *
   */
  public void addArg(GridData arg) {
    args.add(arg);
  }
  
  /**
   *
   */
  public void setArgs(ArrayList<GridData> args) {
    this.args = args;
  }
  
  /**
   *
   */
  public PointFunction getPointFunction() {
    return pf;
  }
  
  /**
   *
   */
  public void setPointFunction(PointFunction pf) {
    this.pf = pf;
  }
  
  public String toString(String indent) {
    return pf.toString(args, indent);
  }
  
  /**
   *
   */
  public String toNoParenString() {
    return pf.toString();
  }
}