/*
 * SDSLProgram.java: This file is part of the SDSLC project.
 *
 * SDSLC: A compiler for high performance stencil computations
 *
 * Copyright (C) 2011-2013 Ohio State University
 *
 * This program can be redistributed and/or modified under the terms
 * of the license specified in the LICENSE.txt file at the root of the
 * project.
 *
 * Contact: P Sadayappan <saday@cse.ohio-state.edu>
 */
package edu.osu.cse.hpcrl.sdsl.ir;



import java.util.ArrayList;

import edu.osu.cse.hpcrl.sdsl.util.CommandLineOptions;
import edu.osu.cse.hpcrl.sdsl.util.OvertileOptions;

/**
 * @author Tom Henretty <henretty@cse.ohio-state.edu>
 */
public class SDSLProgram {
  // Constants
  public static final String INT_TYPE = "int";
  public static final String LONG_TYPE = "long";
  public static final String FLOAT_TYPE = "float";
  public static final String DOUBLE_TYPE = "double";
  
  // Original position in eSDSL source file
  private int sdslIndex = 0;
  
  // Unparsed SDSL string
  private String sdslText;
  
  // Options
  private CommandLineOptions clOptions;
  private OvertileOptions otOptions;

  // FIXME: Change from literal to expression
  //Expression maxIterations;
  long maxIterations;
  
  private ArrayList<Parameter>     params;
  private ArrayList<Constant>      consts;
  private ArrayList<Grid>          grids;
  private ArrayList<GridData>      gridData;
  private ArrayList<PointFunction> pointFuncs;
  private Iteration                iteration;
  
  //////////////////////////////////////////////////////////////////////////////
  // Builder methods
  //////////////////////////////////////////////////////////////////////////////
  
  // Constructors
  /**
   *
   */
  public SDSLProgram() {
    super();
    //maxIterations = null;
    maxIterations = 1;
   
    params      = new ArrayList<Parameter>();
    consts      = new ArrayList<Constant>();
    grids       = new ArrayList<Grid>();
    gridData    = new ArrayList<GridData>();
    pointFuncs = new ArrayList<PointFunction>();
    iteration   = null;
    //stencils   = new ArrayList<Stencil>();
    //reductions = new ArrayList<Reduction>();
  }
  
  
  public String getSDSLText() {
    return sdslText;
  }

  public void setSDSLText(String sdslText) {
    this.sdslText = sdslText;
  }
  
  /**
   *
   */
  //public void setMaxIterations(Expression maxIterations) {
  public void setMaxIterations(long maxIterations) {
     this.maxIterations = maxIterations;
   }
   
  /**
   *
   */
  //public Expression getMaxIterations() {
   public long getMaxIterations() {
     return this.maxIterations;
   }
  
  /**
   *
   */
   public void setIteration(Iteration i) {
     this.iteration = i;
     this.maxIterations = iteration.getMaxIterations();
   }
   
  /**
   *
   */
   public Iteration getIteration() {
     return this.iteration;
   }
   
  // Parameters and constants
  /**
   *
   */
   public void addParam(Parameter p) {
     params.add(p);
   }
   
  /**
   *
   */
   public Parameter getParam(String id) {
     Parameter ret = null;
     for (Parameter p : params) {
       if (p.getName().equals(id)) {
         ret = p;
       }
     }  
     return ret;
   }
   
  /**
   *
   */
   public boolean isParam(String id) {
     boolean ret = false;
     for (Parameter p : params) {
       if (p.getName().equals(id)) {
         ret = true;
       }
     }  
     return ret;
   }
   
  /**
   *
   */
   public ArrayList<Parameter> getParams() {
     return params;
   }
   
  /**
   *
   */
  public void addConst(Constant c) {
    consts.add(c);
  }
   
  /**
   *
   */
   public Constant getConst(String id) {
     Constant ret = null;
     for (Constant c : consts) {
       if (c.getName().equals(id)) {
         ret = c;
       }
     }  
     return ret;
   }
   
  /**
   *
   */
   public boolean isConst(String id) {
     boolean ret = false;
     for (Constant c : consts) {
       if (c.getName().equals(id)) {
         ret = true;
       }
     }  
     return ret;
   }
  
  /**
   *
   */
  public ArrayList<Constant> getConsts() {
    return consts;
  }

  
  // Grids / data / functions
  /**
   *
   */
  public void addGrid(Grid g) {
    grids.add(g);
  }
  
  /**
   *
   */
  public Grid getGrid(String id) {
     Grid ret = null;
     for (Grid g : grids) {
       if (g.getName().equals(id)) {
         ret = g;
       }
     }  
     return ret;
  }
  
  /**
   *
   */
  public ArrayList<Grid> getGrids() {
    return grids;
  }
   
  /**
   *
   */
  public void addGridData(GridData gd) {
    gridData.add(gd);
  }
  
  /**
   *
   */
  public ArrayList<GridData> getGridData() {
    return gridData;
  }
   
  /**
   *
   */
  public void addPointFunction(PointFunction pf) {
    pointFuncs.add(pf);
  }
  
  /**
   *
   */
  public ArrayList<PointFunction> getPointFunctions() {
    return pointFuncs;
  }
  
  /**
   *
   */
  public PointFunction getPointFunction(String name) {
    PointFunction ret = null;
    for (PointFunction pf : pointFuncs) {
      if (pf.getName().equals(name)) {
        ret = pf;
      }
    }
    return ret;
  }


  /**
   * @return the clOptions
   */
  public final CommandLineOptions getCLOptions() {
    return clOptions;
  }


  /**
   * @param clOptions the clOptions to set
   */
  public final void setCLOptions(CommandLineOptions clOptions) {
    this.clOptions = clOptions;
  }


  /**
   * @return the otOptions
   */
  public final OvertileOptions getOTOptions() {
    return otOptions;
  }


  /**
   * @param otOptions the otOptions to set
   */
  public final void setOTOptions(OvertileOptions otOptions) {
    this.otOptions = otOptions;
  }


  public int getSDSLIndex() {
    return sdslIndex;
  }


  public void setSDSLIndex(int sdslIndex) {
    this.sdslIndex = sdslIndex;
  }
}