/*
 * BasicFileIO.java: This file is part of the SDSLC project.
 *
 * SDSLC: A compiler for high performance stencil computations
 *
 * Copyright (C) 2011-2013 Ohio State University
 *
 * This program can be redistributed and/or modified under the terms
 * of the license specified in the LICENSE.txt file at the root of the
 * project.
 *
 * Contact: P Sadayappan <saday@cse.ohio-state.edu>
 */
/**
 * 
 */
package edu.osu.cse.hpcrl.sdsl.util;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;

/**
 * Helper methods for file input / output.
 * 
 * @author Tom Henretty <henretty@cse.ohio-state.edu>
 *
 */
public final class BasicFileIO {

  /**
   * Reads the contents of a text file into a string.
   * 
   * @param fileName Path to the file to be read
   * @return Contents of fileName as a String
   * @throws IOException Thrown if there are any problems reading the file
   */
  public static String readFile(String fileName) throws IOException {
    StringBuffer contents = new StringBuffer();
    BufferedReader br = new BufferedReader(new FileReader(fileName));
    try {
      String line;
      while ((line = br.readLine()) != null) {
        contents.append(line);
        contents.append("\n");
      }
    } finally {
      br.close();
    }
    return contents.toString();
  }
  
  /**
   * Writes the contents of a String to a file. This method will clobber any 
   * file that exists at the given path.
   * 
   * @param fileName Path to the file to be written
   * @param contents String to be written to the file
   * @throws IOException Thrown if there are any problems writing the file
   */
  public static void writeFile(String fileName, String contents) throws IOException {
    Writer w = new BufferedWriter(new FileWriter(fileName));
    try {
      w.write(contents);
    } finally {
      w.close();
    }
  }

}
